@@ -9,7 +9,7 @@
 use strict; use warnings;
 package POE::Component::SSLify::ClientHandle;
 BEGIN {
-  $POE::Component::SSLify::ClientHandle::VERSION = '1.003';
+  $POE::Component::SSLify::ClientHandle::VERSION = '1.004';
 }
 BEGIN {
   $POE::Component::SSLify::ClientHandle::AUTHORITY = 'cpan:APOCAL';
@@ -21,12 +21,11 @@ BEGIN {
 use Net::SSLeay 1.36 qw( die_now die_if_ssl_error );
 
 # We inherit from ServerHandle
-require POE::Component::SSLify::ServerHandle;
-our @ISA = qw( POE::Component::SSLify::ServerHandle );
+use parent 'POE::Component::SSLify::ServerHandle';
 
 # Override TIEHANDLE because we create a CTX
 sub TIEHANDLE {
-	my ( $class, $socket, $version, $options, $ctx ) = @_;
+	my ( $class, $socket, $version, $options, $ctx, $connref ) = @_;
 
 	# create a context, if necessary
 	if ( ! defined $ctx ) {
@@ -43,7 +42,7 @@ sub TIEHANDLE {
 	# die_if_ssl_error won't die on non-blocking errors. We don't need to call connect()
 	# again, because OpenSSL I/O functions (read, write, ...) can handle that entirely
 	# by self (it's needed to connect() once to determine connection type).
-	my $resp = Net::SSLeay::connect( $ssl ) or die_if_ssl_error( 'ssl connect' );
+	my $res = Net::SSLeay::connect( $ssl ) or die_if_ssl_error( 'ssl connect' );
 
 	my $self = bless {
 		'ssl'		=> $ssl,
@@ -51,6 +50,8 @@ sub TIEHANDLE {
 		'socket'	=> $socket,
 		'fileno'	=> $fileno,
 		'client'	=> 1,
+		'status'	=> $res,
+		'on_connect'	=> $connref,
 	}, $class;
 
 	return $self;
@@ -62,13 +63,17 @@ sub TIEHANDLE {
 __END__
 =pod
 
+=for :stopwords Apocalypse
+
+=encoding utf-8
+
 =head1 NAME
 
 POE::Component::SSLify::ClientHandle - Client-side handle for SSLify
 
 =head1 VERSION
 
-  This document describes v1.003 of POE::Component::SSLify::ClientHandle - released February 28, 2011 as part of POE-Component-SSLify.
+  This document describes v1.004 of POE::Component::SSLify::ClientHandle - released March 08, 2011 as part of POE-Component-SSLify.
 
 =head1 DESCRIPTION
 
@@ -82,11 +87,11 @@ Please see those modules/websites for more information related to this module.
 
 =item *
 
-L<POE::Component::SSLify>
+L<POE::Component::SSLify|POE::Component::SSLify>
 
 =item *
 
-L<POE::Component::SSLify::ServerHandle>
+L<POE::Component::SSLify::ServerHandle|POE::Component::SSLify::ServerHandle>
 
 =back
 
@@ -103,5 +108,28 @@ the same terms as the Perl 5 programming language system itself.
 
 The full text of the license can be found in the LICENSE file included with this distribution.
 
+=head1 DISCLAIMER OF WARRANTY
+
+BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
+FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT
+WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER
+PARTIES PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND,
+EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
+IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE
+SOFTWARE IS WITH YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME
+THE COST OF ALL NECESSARY SERVICING, REPAIR, OR CORRECTION.
+
+IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
+WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
+REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE LIABLE
+TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL, OR
+CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE
+SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
+RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
+FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
+SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
+DAMAGES.
+
 =cut
 
